import os
import pandas as pd
import aerosol.functions as af
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
from scipy.stats import norm
from scipy.optimize import curve_fit

# Set global font size for plots
plt.rcParams.update({
    'font.size': 30,
    'axes.titlesize': 30,
    'axes.labelsize': 30,
    'xtick.labelsize': 30,
    'ytick.labelsize': 30,
    'legend.fontsize': 30
})

# Seasonal colors
seasonal_colors = {
    'winter': 'blue',
    'spring': 'magenta',
    'summer': 'green',
    'autumn': 'orange'
}

# Function to get season based on month
def get_season(month):
    if month in [12, 1, 2]:
        return 'winter'
    elif month in [3, 4, 5]:
        return 'spring'
    elif month in [6, 7, 8]:
        return 'summer'
    elif month in [9, 10, 11]:
        return 'autumn'

# Read and process particle data (already in Tomsk time)
df = pd.read_csv("all_particle_data_1h.csv", index_col=0, parse_dates=True)
df = df.iloc[:, df.columns.values.astype(float) >= 2e-9]
conc = af.calc_conc(df, 2.5e-9, 5e-9)

file_path = '/home/demakova/Documents/Tomsk2020/CS_dps_corrected_2020.csv'
df_cs = pd.read_csv(file_path, parse_dates=[0], date_parser=lambda x: pd.to_datetime(x, format='%Y-%m-%d %H:%M:%S'))

file_path = '/home/demakova/Documents/Tomsk2020/CS_dps_corrected_2021.csv'
df_cs2 = pd.read_csv(file_path, parse_dates=[0], date_parser=lambda x: pd.to_datetime(x, format='%Y-%m-%d %H:%M:%S'))

# Merge the two DataFrames
merged_df_cs = pd.concat([df_cs, df_cs2], ignore_index=True)

CS_datetime = merged_df_cs.iloc[:, 0].to_numpy()
CS = merged_df_cs.iloc[:, 1].to_numpy()

df_CS = merged_df_cs
# Process CS data
CS_datetime = df_CS.iloc[:, 0]
CS_values = df_CS.iloc[:, 1].values.flatten()  # Ensure 1D
CS_df = pd.DataFrame({'datetime': CS_datetime, 'CS_concentration': CS_values})
CS_df.set_index('datetime', inplace=True)

# Filter data for hours between 11:00 and 14:00 (Tomsk time)
CS_filtered = CS_df.between_time('00:00', '23:59')

# Calculate daily median of CS concentration
CS_daily = CS_filtered.resample('D').median()

# Resample particle data to daily max values
conc = conc.squeeze()  # Flatten to 1D if necessary
daily_rank = conc.resample("1D").max().dropna()

# Align CS data and particle data
aligned_data = pd.DataFrame({
    'CS_concentration': CS_daily['CS_concentration'],
    'dN': daily_rank
}).dropna()



## Plotting
plt.figure(figsize=(12, 8))

# Determine alpha based on the season
season_alpha = aligned_data.index.month.map(lambda x: 1 if get_season(x) == 'spring' else 1)

lo_hi_percentiles = aligned_data["dN"].quantile((0.0, 0.4, 0.85, 1.0)).values

# Scatter plot with transparency
sc = sns.scatterplot(
    data=aligned_data,
    x='dN',
    y='CS_concentration',
    hue=aligned_data.index.month.map(get_season),
    palette=seasonal_colors,
    alpha=season_alpha,
    legend=False 
)

# Add vertical shading (from the 0th to 40th percentile as blue, and 85th to 100th percentile as green)
plt.axvspan(10, lo_hi_percentiles[1], color='tab:green', alpha=0.2)  # Green at the top
plt.axvspan(lo_hi_percentiles[2], 100000, color='tab:blue', alpha=0.2)  # Blue at the bottom

# Manually create a discrete colorbar
import matplotlib.colors as mcolors
import matplotlib.cm as cm

# Create a discrete colormap using the seasonal colors
colors = ['blue', 'magenta', 'green', 'orange']  # Order of seasons
cmap = mcolors.ListedColormap(colors)

# Define boundaries for the colorbar
bounds = [0, 1, 2, 3, 4]  # The number of seasons (4), creating 4 sections
norm = mcolors.BoundaryNorm(bounds, cmap.N)

# Add colorbar using the custom colormap
cb = plt.colorbar(cm.ScalarMappable(cmap=cmap, norm=norm), ax=plt.gca(), orientation='vertical', ticks=[0.5, 1.5, 2.5, 3.5])
cb.set_ticklabels(['Winter', 'Spring', 'Summer', 'Autumn'])
#cb.set_label('Season', fontsize=25)

# Add labels and title
plt.xlabel(r"$\Delta N_{2.5-5}$")
plt.ylabel("CS, $s^{-1}$")

# Set logarithmic scale and grid
plt.xscale("log")
plt.yscale("log")
plt.xlim(10, 100000)
plt.grid(True)
#plt.legend(title='Season', fontsize = 15)
plt.tight_layout()
plt.savefig("rank_CS.png", dpi=300, bbox_inches="tight")
plt.show()

from scipy.stats import norm, linregress

# Group data by season
aligned_data['season'] = aligned_data.index.month.map(get_season)

# Initialize lists to store p-values and R-squared values
season_stats = {}

# Loop through each season, perform linear regression, and store results
for season in ['winter', 'spring', 'summer', 'autumn']:
    season_data = aligned_data[aligned_data['season'] == season]
    
    if len(season_data) > 1:  # Ensure there are enough data points for regression
        #slope, intercept, r_value, p_value, std_err = linregress(season_data['dN'], season_data['CS_concentration'])
        slope, intercept, r_value, p_value, std_err = linregress(np.log10(season_data['dN']), np.log10(season_data['CS_concentration']))
       
        r_squared = r_value ** 2
        season_stats[season] = {'p-value': p_value, 'R': r_value}
    else:
        season_stats[season] = {'p-value': None, 'R-squared': None}

# Print p-values and R-squared values for each season
for season, stats in season_stats.items():
    print(f"Season: {season.capitalize()}")
    print(f"  p-value: {stats['p-value']}")
    print(f"  R-value: {stats['R']}")
    print("-" * 30)


# Save aligned_data with necessary columns to a CSV file
aligned_data.reset_index(inplace=True)  # Reset index to include 'datetime' as a column
aligned_data.rename(columns={'index': 'date'}, inplace=True)  # Rename index to 'date'
aligned_data[['date', 'dN', 'CS_concentration']].to_csv("dN_CS.csv", index=False)

print("CSV file saved as 'aligned_data.csv'.")