import pandas as pd
import matplotlib.pyplot as plt
import aerosol.functions as af
from scipy.stats import linregress
import numpy as np

# Define seasonal colors
seasonal_colors = {
    'winter': 'blue',
    'spring': 'magenta',
    'summer': 'green',
    'autumn': 'orange'
}

# Function to determine season
def get_season(month):
    if month in [12, 1, 2]:
        return 'winter'
    elif month in [3, 4, 5]:
        return 'spring'
    elif month in [6, 7, 8]:
        return 'summer'
    else:
        return 'autumn'

# Load the J3_max_day data
j3_data = pd.read_csv("J3_max_day.csv", parse_dates=[0], index_col=0)
j3_data.columns = ['J3']  # Rename second column to 'J3' for easy access

# Load all_particle_data_1h and filter columns with data >= 2e-9
df = pd.read_csv("all_particle_data_1h.csv", index_col=0, parse_dates=True)
df = df.iloc[:, df.columns.values.astype(float) >= 2e-9]
df.index = df.index.tz_localize('UTC').tz_convert('UTC')  # Assuming timezone UTC

# Calculate concentration between 2.5e-9 and 5e-9 using a placeholder function `calc_conc`
conc = af.calc_conc(df, 2.5e-9, 5e-9)  # Assuming `af` and `calc_conc` are defined

# Resample and take the daily max concentration
rank = conc.resample("1D").max()
rank.index = pd.to_datetime(rank.index.strftime("%Y%m%d"))

# Merge J3 data with rank data on dates
merged_data = pd.merge(j3_data, rank, left_index=True, right_index=True)
merged_data.columns = ['J3', 'Rank']

# Add season column to data based on month
merged_data['Season'] = merged_data.index.month.map(get_season)

# Remove rows with NaN values in Rank or J3 for the regression
merged_data = merged_data.dropna()

# Set font size
plt.rcParams.update({'font.size': 18})

# Plot with seasonal colors, Rank on x-axis and J3 on y-axis
plt.figure(figsize=(10, 6))

for season, color in seasonal_colors.items():
    season_data = merged_data[merged_data['Season'] == season]
    
    # Skip empty dataframes
    if season_data.empty:
        continue
    
    # Perform linear regression on log-transformed data for each season
    log_rank = (season_data['Rank'])
    log_j3 = (season_data['J3'])
    
    slope, intercept, r_value, p_value, std_err = linregress(log_rank, log_j3)
    
    # Print R and p values for each season
    print(f"Season: {season.capitalize()}")
    print(f"  R value: {r_value:.4f}")
    print(f"  P value: {p_value:.4e}")
    
    # Scatter plot for the current season
    plt.scatter(season_data['J3'], season_data['Rank'], color=color, label=season.capitalize(), alpha=0.7)

# Configure plot settings
plt.ylabel(r"$\Delta N_{2.5-5}$", fontsize=18)
plt.xlabel(r'$\mathrm{J}_{3}, \, \mathrm{cm}^{-3} \cdot \mathrm{s}^{-1}$', fontsize=18)
plt.xscale('log')
plt.yscale('log')
plt.legend(title="Season", fontsize=14)
plt.grid(True)
plt.tight_layout()
# Save the plot
plt.savefig('J_rank.png', dpi=300)
plt.show()

