import numpy as np
import matplotlib.pyplot as plt
import datetime
import pandas as pd
import calendar
from matplotlib.patches import Patch
from matplotlib.ticker import FormatStrFormatter

# Load data
fin = np.loadtxt('GR_2019_2020_2021_shorttime.csv', delimiter=',', skiprows=1)
GR_datetime = np.array(fin[:, 0:3], dtype='int')
dates = np.array([datetime.datetime(s[0], s[1], s[2]) for s in GR_datetime])

# Convert to DataFrame for easier manipulation
data = {
    'Date': dates,
    'J2pos': np.array(fin[:, 3], dtype='float'),
    'J2neg': np.array(fin[:, 4], dtype='float'),
    'J3pos': np.array(fin[:, 5], dtype='float'),
    'J3neg': np.array(fin[:, 6], dtype='float'),
    'GR2_3pos': np.array(fin[:, 7], dtype='float'),
    'GR2_3neg': np.array(fin[:, 8], dtype='float'),
    'GR3_7pos': np.array(fin[:, 9], dtype='float'),
    'GR3_7neg': np.array(fin[:, 10], dtype='float'),
    'GR7_20pos': np.array(fin[:, 11], dtype='float'),
    'GR7_20neg': np.array(fin[:, 12], dtype='float')
}
df = pd.DataFrame(data)
df['Month'] = df['Date'].dt.month  # Extract month for grouping

# Combine data for totals (except for J2)
df['J3_total'] = df['J3pos'] + df['J3neg']
df['GR2_3_total'] = df['GR2_3pos'] + df['GR2_3neg']
df['GR3_7_total'] = df['GR3_7pos'] + df['GR3_7neg']
df['GR7_20_total'] = df['GR7_20pos'] + df['GR7_20neg']

# Variables for plotting
variables = [
    ('J2pos', 'J2neg'),
    ('J3_total',),
    ('GR2_3_total',),
    ('GR3_7_total',),
    ('GR7_20_total',)
]

# Custom y-axis labels
osi = [
    r'$\mathrm{J}_{2, ion}, \, \mathrm{cm}^{-3} \cdot \mathrm{s}^{-1}$',
    r'$\mathrm{J}_{3, total}, \, \mathrm{cm}^{-3} \cdot \mathrm{s}^{-1}$',
    r'$\mathrm{GR}_{2 - 3, total}, \, \mathrm{nm} \cdot \mathrm{h}^{-1}$',
    r'$\mathrm{GR}_{3 - 7, total}, \, \mathrm{nm} \cdot \mathrm{h}^{-1}$',
    r'$\mathrm{GR}_{7 - 20, total}, \, \mathrm{nm} \cdot \mathrm{h}^{-1}$'
]

# Define legend elements
legend_elements = [
    Patch(facecolor='#FF7F0E', edgecolor='k', label='Positive'),  # Orange
    Patch(facecolor='#1F77B4', edgecolor='k', label='Negative'),  # Blue
    Patch(facecolor='#2CA02C', edgecolor='k', label='Total')      # Green
]

# Plot J2 and J3 (Figure 1)
fig1, axes1 = plt.subplots(nrows=2, ncols=1, figsize=(14, 12))

# Plot J2
pos_var, neg_var = variables[0]
monthly_data_pos = [df[df['Month'] == month][pos_var].dropna() for month in range(1, 13)]
monthly_data_neg = [df[df['Month'] == month][neg_var].dropna() for month in range(1, 13)]
positions = np.array([(month * 4) for month in range(1, 13)])

axes1[0].boxplot(monthly_data_pos, positions=positions - 0.5, widths=0.8, patch_artist=True,
                 boxprops=dict(facecolor="#FF7F0E"))
axes1[0].boxplot(monthly_data_neg, positions=positions + 0.5, widths=0.8, patch_artist=True,
                 boxprops=dict(facecolor="#1F77B4"))
axes1[0].set_ylabel(osi[0], fontsize=30)
axes1[0].set_xticks(positions)
axes1[0].set_xticklabels([calendar.month_abbr[m] for m in range(1, 13)], fontsize=30)
axes1[0].set_ylim(0, 0.07)  # Linear scale, range 0 to 0.07
axes1[0].yaxis.set_major_formatter(FormatStrFormatter('%.2f'))  # Format y-axis ticks
axes1[0].tick_params(axis='y', labelsize=20)  # Increase y-axis font size

# Plot J3
var = variables[1][0]
monthly_data_total = [df[df['Month'] == month][var].dropna() for month in range(1, 13)]

axes1[1].boxplot(monthly_data_total, positions=positions, widths=0.8, patch_artist=True,
                 boxprops=dict(facecolor="#2CA02C"))
axes1[1].set_ylabel(osi[1], fontsize=30)
axes1[1].set_xticks(positions)
axes1[1].set_xticklabels([calendar.month_abbr[m] for m in range(1, 13)], fontsize=30)
axes1[1].set_ylim(0, 10)  # Linear scale, range 0 to 10
axes1[1].yaxis.set_major_formatter(FormatStrFormatter('%.1f'))  # Format y-axis ticks
axes1[1].tick_params(axis='y', labelsize=20)  # Increase y-axis font size

# Add legend to the first figure
fig1.legend(handles=legend_elements[:2] + [legend_elements[2]], loc='upper center', fontsize=30)
plt.tight_layout()
plt.savefig('J2_J3_linear.png', dpi=300)

# Plot GR (Figure 2)
fig2, axes2 = plt.subplots(nrows=3, ncols=1, figsize=(14, 18))

for i, var in enumerate(variables[2:]):  # Only GR datasets
    monthly_data_total = [df[df['Month'] == month][var[0]].dropna() for month in range(1, 13)]
    axes2[i].boxplot(monthly_data_total, positions=positions, widths=1, patch_artist=True,
                     boxprops=dict(facecolor="#2CA02C"))
    axes2[i].set_ylabel(osi[i + 2], fontsize=30)
    axes2[i].set_xticks(positions)
    axes2[i].set_xticklabels([calendar.month_abbr[m] for m in range(1, 13)], fontsize=30)
    axes2[i].set_ylim(0, 16)
    axes2[i].yaxis.set_major_formatter(FormatStrFormatter('%.1f'))  # Format y-axis ticks
    axes2[i].tick_params(axis='y', labelsize=20)  # Increase y-axis font size

# Save the second figure
plt.tight_layout()
plt.savefig('GR_totals.png', dpi=300)
