import os
import pandas as pd
import aerosol.functions as af
import numpy as np
import matplotlib.pyplot as plt
import pytz
import seaborn as sns
import scipy
import matplotlib.colors as mcolors
from scipy.stats import norm
from scipy.optimize import curve_fit

# Set timezone
tz = pytz.FixedOffset(420)

# Set global font size for plots
plt.rcParams.update({
    'font.size': 30,
    'axes.titlesize': 30,
    'axes.labelsize': 30,
    'xtick.labelsize': 30,
    'ytick.labelsize': 30,
    'legend.fontsize': 30
})

# Seasonal colors
seasonal_colors = {
    'winter': 'blue',
    'spring': 'magenta',
    'summer': 'green',
    'autumn': 'orange'
}

# Function to get season based on month
def get_season(month):
    if month in [12, 1, 2]:
        return 'winter'
    elif month in [3, 4, 5]:
        return 'spring'
    elif month in [6, 7, 8]:
        return 'summer'
    elif month in [9, 10, 11]:
        return 'autumn'

# Read and process data
df = pd.read_csv("all_particle_data_1h.csv", index_col=0, parse_dates=True)
df = df.iloc[:, df.columns.values.astype(float) >= 2e-9]
df.index = df.index.tz_localize('UTC').tz_convert(tz)
conc = af.calc_conc(df, 2.5e-9, 5e-9)

rank = conc.resample("1d").max()
rank = rank.dropna()

prctiles = np.linspace(0, 1, 21)
rank_percentiles = rank.quantile(prctiles).values.squeeze()

df_master = pd.DataFrame(index=rank.index, columns=["rank"], data=rank.values)
df_master["prc"] = np.nan
df_master["class"] = np.nan

for i in range(len(rank_percentiles) - 1):
    prc_days = rank.index[(rank.values.squeeze() >= rank_percentiles[i]) & (rank.values.squeeze() <= rank_percentiles[i+1])]
    df_master.loc[prc_days, ["prc"]] = prctiles[i+1] * 100

# Read the CS data
file_path = '/home/demakova/Documents/Classification_gases/h2so4.csv'
df_cs = pd.read_csv(file_path, parse_dates=[0])

CS_datetime = df_cs.iloc[:, 0].to_numpy()
CS = df_cs.iloc[:, 1].to_numpy()

# Resample to daily
fon_CS_dmps_daily = pd.Series(CS, index=CS_datetime).resample('D').median()

# Align the lengths of data
min_length = min(len(df_master['rank']), len(fon_CS_dmps_daily))
df_master = df_master.iloc[:min_length]
fon_CS_dmps_daily = fon_CS_dmps_daily.iloc[:min_length]

# Map months to seasons
month_values = fon_CS_dmps_daily.index.month
seasons = np.array([get_season(month) for month in month_values])

# Map seasons to numerical values for colormap
season_to_num = {'winter': 0, 'spring': 1, 'summer': 2, 'autumn': 3}
season_nums = np.array([season_to_num[season] for season in seasons])

# Define colormap
cmap = mcolors.ListedColormap([seasonal_colors[season] for season in ['winter', 'spring', 'summer', 'autumn']])
bounds = [-0.5, 0.5, 1.5, 2.5, 3.5]
norm = mcolors.BoundaryNorm(bounds, cmap.N)

# Plot
plt.figure(figsize=(12, 10))  # Panel size

# Define alpha transparency for seasons
alpha_values = np.array([1 if season == 'spring' else 0.2 for season in seasons])

# Scatter plot with alpha transparency
scatter1 = plt.scatter(df_master['rank'], fon_CS_dmps_daily, c=season_nums, cmap=cmap, norm=norm, s=70, 
                       label='2020', alpha=alpha_values)

# Axes labels and scale
plt.xlabel(r"$\Delta N_{2.5-5}$")
plt.ylabel("$H_2SO_4$ Proxy (cm$^{-3}$)")

# Set logarithmic scale and grid
plt.xscale("log")
plt.yscale("log")
plt.xlim(10, 100000)
# You can modify limits as per your data range
plt.grid(True)

# Colorbar setup
cbar = plt.colorbar(scatter1, ticks=[0, 1, 2, 3])
cbar.ax.set_yticklabels(['winter', 'spring', 'summer', 'autumn'])

# Tight layout for better spacing
plt.tight_layout()

plt.savefig("rank_h2so4.png", dpi=300, bbox_inches="tight")
# Show plot
plt.show()

